@@ -63,7 +63,7 @@ break    any_:label ; @stmt @token @new @cmpl
 continue any_:label ;  @stmt @token @new @cmpl
 goto ANY:label ;  @stmt @token @new @cmpl
 
-try {:try STT_CATCH:catch {:finally   @stmt  @token @new  @name @cmpl
+try {:try catch.. {:catch {:finally   @stmt  @token @new  @name @cmpl
 catch EXPTN:type VARN:name {:catch   @token @stmt
 finally @token
 throw expr:expt ;   @stmt  @token @new @name @cmpl
