@@ -43,7 +43,7 @@ TPARAM = [
 		  ASMType('OBJ', 'Object*', '''
 	knh_format(ctx, w, METHODN__k, %s, KNH_NULL);'''),
 		  ASMType('addr', 'knhvm_addr_t', '''
-	knh_printf(ctx, w, "->%%p", (((char*)bw) + %s - off));'''),
+	knh_printf(ctx, w, "goto->%%p", (%s - off));'''),
 		  ASMType('c2',   'knhvm_class_t', '''
 	knh_write__type(ctx, w, (knh_type_t)%s);'''), 
 		  ASMType('mn',   'knhvm_methodn_t', '''
@@ -187,6 +187,8 @@ CODE_DATA = [
 
  #  ['diagnose', 'u1'], 
  #  ['stackdump'],
+   ['iadd'], ['isub'], ['ilt'],
+   ['init'], # for code threading 
    ['nop'],
 ]
 
@@ -213,45 +215,42 @@ class ASMCode:
 	def write_ctype(self, f):
 		f.write('''
 typedef struct {
-	knhvm_opcode_t code;''')
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/''')
+		f.write('''
+	knhvm_opcode_t opcode;''')
 		c = 0
 		for a in self.struct[1:]:
 			c += 1
+			if a == "addr" : 
+				f.write('''
+#ifdef KNH_USING_THREADEDCODE
+	void *jumpaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	union {
+		knh_int_t a1;  /* oldname */
+		knhvmc_t *jumppc;
+	};''')
+				continue
 			if PARAMTBL[a].isObject(): continue
 			f.write('''
 	%s a%d;''' % (PARAMTBL[a].ctype, c))
 		c = 0
 		for a in self.struct[1:]:
 			c += 1
+			if a == "addr" : continue
 			if not PARAMTBL[a].isObject(): continue
 			f.write('''
 	%s a%d;''' % (PARAMTBL[a].ctype, c))
 		f.write('''
+	void *nextaddr;
 } %s;
 ''' % self.ctype)
 
 		f.write('''
-typedef struct {
-	knhvm_opcode_t code;''')
-		c = 0
-		for a in self.struct[1:]:
-			c += 1
-			if PARAMTBL[a].isObject(): continue
-			f.write('''
-	%s a%d;''' % (PARAMTBL[a].ctype, c))
-		c = 0
-		for a in self.struct[1:]:
-			c += 1
-			if not PARAMTBL[a].isObject(): continue
-			f.write('''
-	%s a%d;''' % (PARAMTBL[a].ctype, c))
-		f.write('''
-	void *w;
-} %sw;
-
-#define %s  (sizeof(%sw)-sizeof(void*))
-''' % (self.ctype, self.size, self.ctype))
-	
+#define %s  (sizeof(%s)-sizeof(void*))
+''' % (self.size, self.ctype))
 	
 	def asm_argv(self):
 		t = ''
@@ -373,7 +372,7 @@ def write_asm_each(f, ac):
 {
 	if(!knh_Compiler_isStopped(o)) {
 		%s *bw = (%s*)knh_Compiler_asmmalloc(ctx, o, %s);
-		bw->code = %s;''' % (rtype, ac.NAME, ac.asm_argv(), ac.ctype, ac.ctype, ac.size, ac.code))
+		bw->opcode = %s;''' % (rtype, ac.NAME, ac.asm_argv(), ac.ctype, ac.ctype, ac.size, ac.code))
 	c = 1;
 	for a in ac.struct[1:]:
 		f.write('''
@@ -501,7 +500,7 @@ def write_asm_c(f):
 
 def write_label(f):
 	f.write('''
-#define PCT_(pc)    (((knh_asmc_nop_t*)pc)->code)
+#define PCT_(pc)    (((knh_asmc_nop_t*)pc)->opcode)
 ''')
 	write_line(f)
 	write_chapter(f, '[label]')
@@ -605,7 +604,8 @@ void knh_KLRCode_%s__dump(Ctx *ctx, %s *bw, int off, OutputStream *w, Method *mt
 		knh_printf(ctx, w, "[%%d:%%p]", line, ((knhvmc_t*)bw) - off);
 	}
 	knh_putc(ctx, w, ' ');
-	knh_write__s(ctx, w, "%s");'''  % (ac.NAME, ac.ctype, ac.NAME.lower()))
+	knh_write__s(ctx, w, "%s");
+	knh_printf(ctx, w, "(%%d)", bw->opcode); '''  % (ac.NAME, ac.ctype, ac.NAME.lower()))
 	c = 1
 	for a in ac.struct[1:]:
 		f.write('''
@@ -627,20 +627,19 @@ def write_dump(f):
 	write_line(f)
 	CPROTO.append('void knh_KLRCode__dumpmtd(Ctx *ctx, KLRCode *o, OutputStream *w, Method *mtd)')
 	f.write('''
-
 PUBLIC
 void knh_KLRCode__dumpmtd(Ctx *ctx, KLRCode *o, OutputStream *w, Method *mtd)
 {
 	knh_bytes_t vmc = knh_KLRCode_tobytes(o);
 	knhvmc_t *pc = (knhvmc_t*)vmc.buf;
 	knhvmc_t *tail = pc + vmc.len;	
-	int off = (int)pc;
+//	int off = (int)pc;
 	while(pc < tail) {
 		switch(PCT_(pc)) {
 ''');
 	for ac in TCODE:
 		f.write('''\t\tcase %s :
-			knh_KLRCode_%s__dump(ctx, (%s*)pc, off, w, mtd);
+			knh_KLRCode_%s__dump(ctx, (%s*)pc, 0 /*off*/, w, mtd);
 			pc += %s;
 			break;
 ''' % (ac.code, ac.NAME, ac.ctype, ac.size))
@@ -664,28 +663,134 @@ def vm_macro(ac, label):
 	c = 1
 	for a in ac.struct[1:]:
 		if a == 'addr':
-			m += ', pc += ((const %s*)pc)->a%d, %s' % (ac.ctype, c, label)
+			m += ', pc = asmcode->jumppc, %s' % (label)
 		else:
-			m += ', %s_(((const %s*)pc)->a%d)' % (a, ac.ctype, c)
+			m += ', %s_(asmcode->a%d)' % (a, c)
 		c += 1
 	m += ')'
 	return m
 
 def write_exec(f):
 	CPROTO.append('METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)')
+	CPROTO.append('knhvmc_t* KNH_CC_FASTCALL konohavm(Ctx *ctx, knh_sfp_t *sfp, knhvmc_t *pc_start)')
+	CPROTO.append('void knhvmc_codethreading(Ctx *ctx, knhvmc_t *pc, void **codeaddr)')
 	write_chapter(f, '[exec]')
+
 	f.write('''
-#ifndef KNH_CC_LABELPTR
-METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
+/* ------------------------------------------------------------------------ */
+
+void knhvmc_codethreading(Ctx *ctx, knhvmc_t *pc, void **codeaddr)
 {
+	while(1) {
+		knh_asmc_jmp_t *asmc = (knh_asmc_jmp_t*)pc;
+		KNH_ASSERT(asmc->opcode <= ASMC_NOP);
+#ifdef KNH_USING_THREADEDCODE
+		DBG2_P("opcode=%d %p %p", asmc->opcode, &asmc->codeaddr, &asmc->nextaddr);
+#endif/*KNH_USING_THREADEDCODE*/
+		if(asmc->opcode == ASMC_HALT) {
+#ifdef KNH_USING_THREADEDCODE
+			asmc->codeaddr = codeaddr[asmc->opcode];
+#endif/*KNH_USING_THREADEDCODE*/
+			return;
+		}
+#ifdef KNH_USING_THREADEDCODE
+		asmc->codeaddr = codeaddr[asmc->opcode];
+#endif/*KNH_USING_THREADEDCODE*/
+		switch(asmc->opcode) {
+''');
+	for ac in TCODE:
+		if len(ac.struct)>1 and ac.struct[1] == 'addr' :
+			f.write('''\t\tcase %s : {
+			asmc->jumppc = pc + asmc->a1;
+#ifdef KNH_USING_THREADEDCODE
+			asmc->jumpaddr = codeaddr[((knh_asmc_jmp_t*)(asmc->jumppc))->opcode];
+#endif/*KNH_USING_THREADEDCODE*/
+			pc += %s; 
+			break;
+		}
+''' % (ac.code, ac.size))
+		else :
+			f.write('''\t\tcase %s :  {
+			pc += %s; 
+			break;
+		}
+''' % (ac.code, ac.size))
+	f.write('''
+		default :
+			DBG_P("UNKNOWN VMCODE: %d\\n", PCT_(pc));
+			KNH_ASSERT(pc == NULL);
+			return;
+		}
+	}
+}
+''')
+
+	f.write('''
+#ifndef KNH_USING_THREADEDCODE
+METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
+/*knhvmc_t* KNH_CC_FASTCALL konohavm(Ctx *ctx, knh_sfp_t *sfp, knhvmc_t *pc_start)*/
+{
+	static void **globalMachineToJump = NULL;
 	register knhvmc_t *pc = DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
+	//register knhvmc_t *pc = pc_start;
+	
 	L_HEAD:;
 	switch(PCT_(pc)) {
 ''');
 	for ac in TCODE:
 		f.write('''\tcase %s :
+		{
+			const %s *asmcode = (%s*)pc;
 #ifdef DBG_P_VM
-		knh_KLRCode_%s__dump(ctx, (%s*)pc, 0, KNH_STDOUT, KNH_NULL);
+			knh_KLRCode_%s__dump(ctx, asmcode, 0, KNH_STDOUT, KNH_NULL);
+#endif
+#ifdef VM_%s
+			%s;
+#else
+			DBG_P("TODO: VM_%s");
+#endif
+			pc += %s;
+			goto L_HEAD;
+		}
+''' % (ac.code, ac.ctype, ac.ctype, ac.code, ac.type.upper(), vm_macro(ac, 'L_HEAD'), ac.type.upper(), ac.size))
+
+	f.write('''
+	}
+	goto L_HEAD;
+}
+#endif/*KNH_USING_THREADEDCODE*/
+''')
+
+	write_line(f)
+	f.write('''
+#ifdef KNH_USING_THREADEDCODE
+METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
+/* knhvmc_t* KNH_CC_FASTCALL konohavm(Ctx *ctx, knh_sfp_t *sfp, knhvmc_t *pc_start) */
+{
+	static void *globalMachineToJump[] = {''')
+	c = 0
+	for ac in TCODE:
+		f.write('\n\t\t&&L_%s,' % ac.code)
+		c += 1
+#	for n in range(256 - c):
+#		f.write('\n\t\t&&L_ASMC_HALT,')
+
+	f.write('''
+		&&L_ASMC_HALT
+	};
+	
+	register knhvmc_t *pc =  DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
+	//register knhvmc_t *pc =  pc_start;
+	//goto *(((knh_asmc_jmp_t*)pc)->codeaddr);
+	goto *globalMachineToJump[PCT_(pc)]; /* this is needed to init */
+
+''')
+	for ac in TCODE:
+		f.write('''
+	L_%s:; {
+		const %s *asmcode = (%s*)pc;
+#ifdef DBG_P_VMC
+		knh_KLRCode_%s__dump(ctx, asmcode, 0, KNH_STDOUT, KNH_NULL);
 #endif
 #ifdef VM_%s
 		%s;
@@ -693,58 +798,21 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 		DBG_P("TODO: VM_%s");
 #endif
 		pc += %s;
-		break;
-''' % (ac.code, ac.code, ac.ctype, ac.type.upper(), vm_macro(ac, 'L_HEAD'), ac.type.upper(), ac.size))
+		goto *(asmcode->nextaddr);
+	}
+''' % (ac.code, ac.ctype, ac.ctype, ac.NAME, ac.type.upper(), vm_macro(ac, '*(asmcode->jumpaddr)'), ac.type.upper(), ac.size))
 
 	f.write('''
-	}
-	goto L_HEAD;
 }
 #endif
-''')
 
-	write_line(f)
-	f.write('''
-#ifdef KNH_CC_LABELPTR
+/*******
 METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 {
-	static void *globalMachineToJump[] = {''')
-	c = 0
-	for ac in TCODE:
-		f.write('\n\t\t&&L_%s,' % ac.code)
-		c += 1
-	for n in range(256 - c):
-		f.write('\n\t\t&&L_UNKNOWN,')
-
-	f.write('''
-		&&L_UNKNOWN
-	};
-	knhvmc_t *pc =  DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
-	goto *globalMachineToJump[PCT_(pc)];
-
-''')
-	for ac in TCODE:
-		f.write('''
-	L_%s:;
-#ifdef DBG_P_VMC
-	knh_KLRCode_%s__dump(ctx, (%s*)pc, 0, KNH_STDOUT, KNH_NULL);
-#endif
-#ifdef VM_%s
-		%s;
-#else
-		DBG_P("TODO: VM_%s");
-#endif
-	pc += %s;
-	goto *globalMachineToJump[PCT_(pc)];
-''' % (ac.code, ac.NAME, ac.ctype, ac.type.upper(), vm_macro(ac, '*globalMachineToJump[PCT_(pc)]'), ac.type.upper(), ac.size))
-
-	f.write('''
-	L_UNKNOWN:;
-	DBG_P("Broken code??");
-	KNH_ASSERT(pc == NULL);
-	KNH_THROWs(ctx, "Halt!!");
+	konohavm(ctx, sfp, DP((KLRCode*)DP(sfp[-1].mtd)->code)->code);
 }
-#endif
+*******/
+
 ''')
 
 #------------------------------------------------------------------------------
