@@ -1104,11 +1104,19 @@ static
 void KNH_ASM_PPUSH(Ctx *ctx, Compiler *cpr, Term *tm, int level)
 {
 	if(level == 0) {
-		if(IS_Token(tm) && SP((Token*)tm)->tt == TT_SYSCONST) {
-			KNH_ASM_PUT_SYSCONST(ctx, cpr, DP((Token*)tm)->index);
+		if(knh_Compiler_isShifted(cpr)) {
+			if(IS_Token(tm) && SP((Token*)tm)->tt == TT_SYSCONST) {
+				KNH_ASM_PUT_SYSCONST(ctx, cpr, DP((Token*)tm)->index);
+			}
+			else {
+				KNH_ASM_MOVE(ctx, cpr, 0, tm);
+			}
+			return;
 		}
 		else {
-			KNH_ASM_MOVE(ctx, cpr, 0, tm);
+			knh_Compiler_setShifted(cpr, 1);
+			KNH_ASM_PUSH(ctx, cpr, tm);
+			KNH_ASSERT(knh_Compiler_isShifted(cpr));
 		}
 	}
 	else {
@@ -1500,6 +1508,40 @@ void knh_StmtCALL_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_t
 /* ======================================================================== */
 /* [OP] */
 
+static
+int knh_StmtOPX_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level, Method *mtd)
+{
+	knh_class_t cid = DP(mtd)->cid;
+	knh_methodn_t mn = DP(mtd)->mn;
+	if(cid == CLASS_Int) {
+		switch(mn) {
+		case METHODN_opAdd :
+		case METHODN_opSub :
+		case METHODN_opLt :
+			knh_Stmt_terms_cmplpush(ctx, stmt, 1, cpr, ns, NNTYPE_Int, level);
+			knh_Stmt_terms_cmplpush(ctx, stmt, 2, cpr, ns, NNTYPE_Int, level+1);
+			break;
+		default:
+			return 0;
+		}
+		switch(mn) {
+		case METHODN_opAdd :
+			KNH_ASM_IADD(ctx, cpr); break;
+		case METHODN_opSub :
+			KNH_ASM_ISUB(ctx, cpr); break;
+		case METHODN_opLt :
+			KNH_ASM_ILT(ctx, cpr); break;
+		default:
+			KNH_ASSERT(ctx == NULL /* this won't happen */);
+		}
+		return 1;
+	}
+	return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+
+
 void knh_StmtOP_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
 {
 	int i;
@@ -1511,6 +1553,10 @@ void knh_StmtOP_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_typ
 	Method *mtd = DP(mtdtk)->mtd;
 	DEBUG3_ASSERT(IS_Method(mtd));
 
+	if(knh_StmtOPX_cmpl(ctx, stmt, cpr, ns, reqt, level, mtd)) {
+		return;
+	}
+
 	SP(stmt)->stt = STT_CALL; /* for error message  */
 	knh_type_t btype = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[1]);
 	knh_class_t base_cid = TYPE_UNMASK_NN(btype);
