@@ -54,22 +54,16 @@ extern "C" {
 #define knh_Bytes_value(o)     (o)->buf
 #define knh_Bytes_last(o)      ((o)->buf + (o)->size)
 #define knh_Bytes_tochar(b)  (char*)knh_Bytes_value(b)
-#define knh_BytesConv_struct_copy NULL
-#define knh_BytesConv_struct_compare NULL
 #define KNH_SIZE(v)         knh_size(v)
 #define B(c)     new_bytes(c)
 #define B2(c,n)   new_bytes__2(c,n)
 #define STEXT(c)  new_bytes__2(c,sizeof(c)-1)
 #define ISB(t,c) (t.len == (sizeof(c)-1) && knh_strncmp((char*)t.buf,c,t.len) == 0)
+#define knh_BytesConv_struct_copy NULL
+#define knh_BytesConv_struct_compare NULL
 #define CLASS_newid          ((knh_class_t)-1)
 #define CLASS_unknown        ((knh_class_t)-2)
 #define knh_Class_cid(c)     (knh_class_t)(c)->cid
-#define knh_ClassMap_struct_copy    NULL
-#define knh_ClassMap_struct_compare  NULL
-#define knh_tstruct_isNative(sid)   (sid < KONOHA_TSTRUCT_SIZE)
-#define knh_ClassStruct_struct_copy    NULL
-#define knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
-#define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
 #define IS_SPEC(o)   (CLASS_IntUnit <= knh_Object_cid(o)  && knh_Object_cid(o) <= CLASS_Vocabulary)
 #define KNH_FLAG_TF_NN                KNH_FLAG_T0
 #define TYPE_ISNULLABLE(t)            ((t & KNH_FLAG_TF_NN)==0)
@@ -84,6 +78,12 @@ extern "C" {
 #define TYPE_any                      CLASS_Any
 #define NNTYPE_any                    CLASS_any
 #define TYPEN(type)  knh_TYPEN(ctx,type)
+#define knh_ClassMap_struct_copy    NULL
+#define knh_ClassMap_struct_compare  NULL
+#define knh_tstruct_isNative(sid)   (sid < KONOHA_TSTRUCT_SIZE)
+#define knh_ClassStruct_struct_copy    NULL
+#define knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
+#define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
 #define knh_Connection_struct_copy    NULL
 #define knh_Connection_struct_compare  NULL
 #define KNH_STDIN          (((Context*)ctx)->in)
@@ -94,6 +94,16 @@ extern "C" {
 #define knh_Exception_struct_copy   NULL
 #define knh_Exception_struct_compare  NULL
 #define new_Exception__s(ctx, s)     new_Exception__b(ctx, B(s))
+#define KNH_PERRNO(ctx, emsg, func) { \
+		knh_perrno(ctx, emsg, func, __FILE__, __LINE__); \
+	}\
+
+
+#define KNH_THROW_OUTOFINDEX(ctx, n, max) { \
+		knh_throw_OutOfIndex(ctx, n, max, __FILE__, __LINE__); \
+	}\
+
+
 #define KNH_SETJUMP(hdlr)   setjmp(DP((ExceptionHandler*)hdlr)->jmpbuf)
 #define knh_ExceptionHandler_struct_copy   NULL
 #define knh_ExceptionHandler_struct_compare  NULL
@@ -222,8 +232,8 @@ extern "C" {
 #define knh_tMethodField_add(ctx, mf)       knh_Array_add(ctx, DP(knh_rootSystem)->tMethodFields, UP(mf))
 #define knh_tMethodField(n)                 (MethodField*)knh_Array_n(DP(knh_rootSystem)->tMethodFields, n)
 #define knh_tMethodField_size               knh_Array_size(DP(knh_rootSystem)->tMethodFields)
-#define new_Int(ctx, n)      new_Int__fast(ctx, CLASS_Int, (n))
-#define new_Float(ctx, n)      new_Float__fast(ctx, CLASS_Float, (n))
+#define new_Int(ctx, n)      new_Int__fast(ctx, (n))
+#define new_Float(ctx, n)      new_Float__fast(ctx, (n))
 #define new_String__T(ctx, text)    new_StringX__T(ctx, CLASS_String, text)
 #define KNH_FLAG_SF_FIELD          KNH_FLAG_T1
 #define STRUCT_ISFIELD(sid)        ((sid & KNH_FLAG_SF_FIELD) == KNH_FLAG_SF_FIELD)
@@ -275,6 +285,8 @@ extern "C" {
 /* ======================================================================== */
 /* [PROTOTYPE] */
 KNHAPI(void) konoha_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift);
+KNHAPI(knh_Array_t*) new_Array(Ctx *ctx, knh_class_t p1, size_t capacity);
+KNHAPI(void) knh_Array_add(Ctx *ctx, Array *o, Any *value);
 KNHAPI(Bytes*) new_Bytes(Ctx *ctx, size_t capacity);
 KNHAPI(knh_bytes_t) knh_Bytes_tobytes(Bytes *o);
 KNHAPI(void) knh_Bytes_clear(Bytes *o);
@@ -282,9 +294,25 @@ KNHAPI(void) knh_Bytes_putc(Ctx *ctx, Bytes *o, int ch);
 KNHAPI(void) knh_Bytes_write(Ctx *ctx, Bytes *o, knh_bytes_t v);
 KNHAPI(knh_bytes_t) new_bytes(char *c);
 KNHAPI(knh_bytes_t) new_bytes__2(char *buf, size_t len);
+KNHAPI(int) knh_bytes_strcmp(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(int) knh_bytes_strcasecmp(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(knh_bool_t) knh_bytes_equals(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(knh_bool_t) knh_bytes_equalsIgnoreCase(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(knh_bool_t) knh_bytes_startsWith(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(knh_bool_t) knh_bytes_endsWith(knh_bytes_t v1, knh_bytes_t v2);
+KNHAPI(knh_index_t) knh_bytes_index(knh_bytes_t v, knh_int_t ch);
+KNHAPI(knh_index_t) knh_bytes_rindex(knh_bytes_t v, knh_int_t ch);
+KNHAPI(knh_bytes_t) knh_bytes_first(knh_bytes_t t, knh_int_t loc);
+KNHAPI(knh_bytes_t) knh_bytes_last(knh_bytes_t t, knh_int_t loc);
+KNHAPI(knh_bytes_t) knh_bytes_skipscheme(knh_bytes_t t);
+KNHAPI(char*) knh_format_bytes(char *buf, size_t bufsiz, knh_bytes_t t);
+KNHAPI(knh_int_t) knh_bytes_toint(knh_bytes_t t);
+KNHAPI(knh_float_t) knh_bytes_tofloat(knh_bytes_t t);
+KNHAPI(knh_int64_t) knh_bytes_toint64(knh_bytes_t t);
 KNHAPI(Exception*) new_Exception(Ctx *ctx, String *msg);
 KNHAPI(Exception*) new_Exception__b(Ctx *ctx, knh_bytes_t msg);
-KNHAPI(Exception*) new_Exception__errno(Ctx *ctx);
+KNHAPI(void) knh_perrno(Ctx *ctx, char *emsg, char *func, char *file, int line);
+KNHAPI(void) knh_throw_OutOfIndex(Ctx *ctx, knh_integer_t n, size_t max, char *file, int line);
 KNHAPI(ClassSpec*) new_Unit(Ctx *ctx, char *tag, knh_float_t min, knh_float_t max, knh_float_t defv, knh_float_t step);
 KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file);
 KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t e);
@@ -303,26 +331,31 @@ KNHAPI(void) knh_write_BOL(Ctx *ctx, OutputStream *w);
 KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Any *x, Any *m);
 KNHAPI(void) knh_printf(Ctx *ctx, OutputStream *w, char *fmt, ...);
 KNHAPI(void) konoha_says(Ctx *ctx, int type, char *fmt, ...);
-KNHAPI(ResultSet*) new_ResultSet(Ctx *ctx, knh_dbcur_t *dbcur, size_t column_size);
+KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, ResultSet *o, size_t column_size);
 KNHAPI(void) knh_ResultSet_setName(Ctx *ctx, ResultSet *o, size_t n, String *name);
+KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, ResultSet *o);
 KNHAPI(void) knh_ResultSet_setInt(Ctx *ctx, ResultSet *o, size_t n, knh_integer_t value);
 KNHAPI(void) knh_ResultSet_setFloat(Ctx *ctx, ResultSet *o, size_t n, knh_float_t value);
 KNHAPI(void) knh_ResultSet_setText(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t t);
-KNHAPI(void) knh_ResultSet_setBLOB(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t t);
+KNHAPI(void) knh_ResultSet_setBlob(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t t);
 KNHAPI(void) knh_ResultSet_setNULL(Ctx *ctx, ResultSet *o, size_t n);
 KNHAPI(void) konoha_addAliasURN(Ctx *ctx, char *alias, char *urn);
+KNHAPI(String*) new_String(Ctx *ctx, knh_bytes_t t, String *orign);
+KNHAPI(knh_bytes_t) knh_String_tobytes(String *o);
+KNHAPI(knh_bytes_t) knh_StringNULL_tobytes(String *o, knh_bytes_t def);
+KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv);
 KNHAPI(void) konoha_addParserDriver(Ctx *ctx, char *alias, knh_parser_drvapi_t *d);
 KNHAPI(ClassSpec*) new_Vocabulary(Ctx *ctx, char *lang, char *defval, ...);
 KNHAPI(void) konoha_addVocabularyMapper(Ctx *ctx, knh_class_t scid, char *text);
-KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv);
 KNHAPI(void) konoha_addDBDriver(Ctx *ctx, char *alias, knh_db_drvapi_t *d);
 KNHAPI(void) konoha_addIODriver(Ctx *ctx, char *alias, knh_iodrv_t *d);
 KNHAPI(void) konoha_addRegexDriver(Ctx *ctx, char *alias, knh_regex_drvapi_t *d);
 KNHAPI(size_t) knh_stat_usedMemorySize(void);
 KNHAPI(knh_ftraverse) konoha_sweep();
 KNHAPI(int) konoha_issweep(knh_ftraverse ftr);
-KNHAPI(Int*) new_Int__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value);
-KNHAPI(Float*) new_Float__fast(Ctx *ctx, knh_class_t cid, knh_float_t value);
+KNHAPI_(Int*) new_Int__fast(Ctx *ctx, knh_integer_t value);
+KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value);
+KNHAPI_(Float*) new_Float__fast(Ctx *ctx, knh_float_t value);
 KNHAPI(knh_boolean_t) knh_System_hasLibrary(Ctx *ctx, char *libname, char *funcname);
 KNHAPI(void) konoha_addClassSpecFunc(Ctx *ctx, char *urn, f_makespec func);
 KNHAPI(int) konoha_setopt(int argc, char **argv);
@@ -333,6 +366,7 @@ KNHAPI(Ctx*) konoha_open_context(konoha_t konoha);
 KNHAPI(void) konoha_close_context(konoha_t konoha, Ctx *ctx);
 KNHAPI(void) konoha_read(Ctx *ctx, char *fpath);
 KNHAPI(int) konoha_main(Ctx *ctx, int argc, char **argv);
+KNHAPI(char*) knh_format_ospath(Ctx *ctx, char *buf, size_t bufsiz, knh_bytes_t path);
 KNHAPI(void) konoha_SETv(Ctx *ctx, Object **v, Object *o);
 KNHAPI(void) konoha_FINALv(Ctx *ctx, Object **v);
 KNHAPI(knh_class_t) konoha_addClass(Ctx *ctx, knh_flag_t flag, char *name, int method_size);
