@@ -38,7 +38,7 @@
 #define KONOHA_VERSION   PACKAGE_VERSION
 #else
 #define KONOHA_NAME     "konoha"
-#define KONOHA_VERSION  "0.2.3"
+#define KONOHA_VERSION  "0.2.4"
 #endif
 
 #ifndef KNH_DBGMODE2
@@ -146,7 +146,7 @@
 
 #ifdef KNH_FASTMODE
 #define KNH_USING_FASTMALLOC 1
-#define KNH_USING_WORDKLRC   1
+//#define KNH_USING_WORDKLRC   1
 #endif
 
 /* ======================================================================== */
