@@ -359,7 +359,7 @@ typedef struct knh_Object_t {
 #define ARG_integer(a)     ((a.i)->value)
 #define ARG_uinteger(a)    ((knh_uinteger_t)(a.i)->value)
 
-#define ARG_float(a)       ((float)(a.f)->value)
+#define ARG_float(a)       ((knh_float_t)(a.f)->value)
 #define ARG_double(a)      ((double)(a.f)->value)
 #define ARG_char(a)        (knh_String_tochar(a.s))
 #define ARG_bytes(a)       (knh_Bytes_tobytes(a.s))
