@@ -53,7 +53,7 @@ KNHAPI(size_t) knh_stat_usedMemorySize(void)
 
 /* ------------------------------------------------------------------------ */
 
-INLINE void *knh_malloc(Ctx *ctx, size_t size)
+void *knh_malloc(Ctx *ctx, size_t size)
 {
 #ifdef KONOHA_OS__TB
 	if(size == 0) size=4;
@@ -71,7 +71,7 @@ INLINE void *knh_malloc(Ctx *ctx, size_t size)
 
 /* ------------------------------------------------------------------------ */
 
-INLINE void knh_free(void *block, size_t size)
+void knh_free(Ctx *ctx, void *block, size_t size)
 {
 #ifdef KNH_STATMODE0
 	knh_statUsedMemorySize -= size;
@@ -99,7 +99,7 @@ void *DBG2_malloc(Ctx *ctx, size_t size)
 
 /* ------------------------------------------------------------------------ */
 
-void DBG2_free(void *p, size_t size)
+void DBG2_free(Ctx *ctx, void *p, size_t size)
 {
 	size_t *block = ((size_t*)p) - 1;
 #ifdef KNH_STATMODE0
@@ -186,10 +186,96 @@ void knh_tObject_free(knh_Object_t *t)
 		} )
 #endif
 	}
-	KNH_FREE(t, SIZEOF_TOBJECT);
+	KNH_FREE(NULL, t, SIZEOF_TOBJECT);
 	t = NULL;
 }
 
+/* ------------------------------------------------------------------------ */
+/* [fastmalloc] */
+
+void *knh_fastmalloc(Ctx *ctx, size_t size)
+{
+	if(size <= (sizeof(knh_Object_t) - sizeof(knh_int_t))) {
+		knh_Object_t *o;
+		if(ctx == NULL) {
+			if(knh_UnusedObject == NULL) {
+				KNH_ASSERT(knh_UnusedObjectSize == 0);
+				knh_UnusedObject = new_UnusedObject(NULL);
+				if(knh_UnusedObject == NULL) {
+					KNH_EXIT("Enlarge KNH_TOBJECTLIST_SIZE %d x %d",
+							KNH_TOBJECTLIST_SIZE, KNH_TOBJECT_SIZE);
+					return NULL;
+				}
+				else {
+					knh_UnusedObjectSize += KNH_TOBJECT_SIZE;
+				}
+			}
+			o = knh_UnusedObject;
+			knh_UnusedObject = (knh_Object_t*)o->ref;
+			knh_UnusedObjectSize -= 1;
+		}
+		else {
+			if(ctx->unusedObject == NULL) {
+				KNH_ASSERT(ctx->unusedObjectSize == 0);
+				((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+				if(ctx->unusedObject == NULL) {
+					KNH_EXIT("Enlarge KNH_TOBJECTLIST_SIZE %d x %d",
+							KNH_TOBJECTLIST_SIZE, KNH_TOBJECT_SIZE);
+					return NULL;
+				}
+				else {
+					((knh_Context_t*)ctx)->unusedObjectSize += KNH_TOBJECT_SIZE;
+				}
+			}
+			o = ctx->unusedObject;
+			((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
+			((knh_Context_t*)ctx)->unusedObjectSize -= 1;
+		}
+		o->h.magic = KNH_FASTMALLOC_MAGIC;
+		knh_int_t *p = (knh_int_t*)o;
+		return (void*)(p+1);
+	}
+	else {
+		void *block = malloc(size);
+		if (block == NULL) {
+			KNH_EXIT("OutOfMemory!!: %d bytes used", (int)knh_statUsedMemorySize);
+			//KNH_THROWs(ctx, "OutOfMemory!!");
+		}
+#ifdef KNH_STATMODE0
+		knh_statUsedMemorySize += size;
+#endif
+		return block;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_fastfree(Ctx *ctx, void *block, size_t size)
+{
+	if(size <= (sizeof(knh_Object_t) - sizeof(knh_int_t))) {
+		knh_Object_t *o = (knh_Object_t*)(((knh_int_t*)block)-1);
+		assert(o->h.magic == KNH_FASTMALLOC_MAGIC);
+		o->h.magic = 0;
+		if(ctx == NULL) {
+			o->ref = knh_UnusedObject;
+			knh_UnusedObject = o;
+			knh_UnusedObjectSize += 1;
+		}
+		else {
+			o->ref = ctx->unusedObject;
+			((knh_Context_t*)ctx)->unusedObject = o;
+			((knh_Context_t*)ctx)->unusedObjectSize += 1;
+		}
+	}
+	else {
+#ifdef KNH_STATMODE0
+		knh_statUsedMemorySize -= size;
+#endif
+		free(block);
+	}
+}
+
+
 /* ------------------------------------------------------------------------ */
 /* [PObject] */
 
@@ -424,14 +510,14 @@ void knh_Object_RCsweep(Ctx *ctx, Object *o)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(f_traverse) konoha_sweep()
+KNHAPI(knh_ftraverse) konoha_sweep()
 {
 	return knh_Object_RCsweep;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(int) konoha_issweep(f_traverse ftr)
+KNHAPI(int) konoha_issweep(knh_ftraverse ftr)
 {
 	return _IS_SWEEP(ftr);
 }
@@ -439,7 +525,7 @@ KNHAPI(int) konoha_issweep(f_traverse ftr)
 /* ------------------------------------------------------------------------ */
 
 static INLINE
-void knh_fields_traverse(Ctx *ctx, Object **a, size_t capacity, f_traverse ftr)
+void knh_fields_traverse(Ctx *ctx, Object **a, size_t capacity, knh_ftraverse ftr)
 {
 	size_t i = 0;
 	for(i = 0; i < capacity; i++) {
@@ -479,12 +565,20 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 				for(i = 0; i < bsize; i++) {
 					knh_Object_RCsweep(ctx, v[i]);
 				}
-				KNH_FREE(o->ref, bsize * sizeof(Object*));
+				KNH_FREE(ctx, o->ref, bsize * sizeof(Object*));
 			}
 			o->ref = NULL;
 		}
 		break;
 
+	case CLASS_Any:
+		{
+			knh_COBJ_t *c = (knh_COBJ_t*)o;
+			c->cfree(ctx, c->cptr, knh_Object_RCsweep);
+			c->cptr = NULL;
+		}
+		break;
+
 	case CLASS_Nue :
 		{
 			knh_Nue_t *v = (knh_Nue_t*)o;
@@ -513,7 +607,7 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 				knh_Object_RCsweep(ctx, UP(s->orign));
 			}
 			else if(!knh_String_isTextSgm(o)) {
-				KNH_FREE(s->str, KNH_SIZE(s->size + 1));
+				KNH_FREE(ctx, s->str, KNH_SIZE(s->size + 1));
 				s->str = NULL;
 			}
 		}
@@ -522,7 +616,7 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 	case CLASS_Bytes :
 		{
 			knh_Bytes_t *b = (knh_Bytes_t*)o;
-			knh_barray_free(b->buf);
+			knh_barray_free(ctx, b->buf);
 			b->buf = NULL;
 		}
 		break;
@@ -550,6 +644,14 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 		}
 		break;
 
+	case CLASS_DictIdx :
+		{
+			knh_DictIdx_t *d = (knh_DictIdx_t*)o;
+			knh_Object_RCsweep(ctx, UP(d->terms));
+			knh_Object_RCsweep(ctx, UP(d->termsDictSet));
+		}
+		break;
+
 	case CLASS_Regex :
 		{
 			knh_Regex_traverse(ctx, (knh_Regex_t*)o, knh_Object_RCsweep);
@@ -602,10 +704,10 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 //				DBG2_P("sid=%d,%s size=%d,%d", o->h.bcid, STRUCTN(o->h.bcid), size, knh_tClass[o->h.cid].size);
 //			}
 			DBG_(KNH_ASSERT(size == knh_tClass[o->h.cid].size);)
-			if(knh_tClass[o->h.cid].size == 0) {
-				DBG2_P("sid=%d,%s size=%d,%d", o->h.bcid, STRUCTN(o->h.bcid), size, knh_tClass[o->h.cid].size);
-			}
-			KNH_FREE(o->ref, knh_tClass[o->h.cid].size);
+//			if(knh_tClass[o->h.cid].size == 0) {
+//				DBG2_P("sid=%d,%s size=%d,%d", o->h.bcid, STRUCTN(o->h.bcid), size, knh_tClass[o->h.cid].size);
+//			}
+			KNH_FREE(ctx, o->ref, knh_tClass[o->h.cid].size);
 			o->ref = NULL;
 		}
 	}
@@ -631,7 +733,7 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, f_traverse ftr)
+void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, knh_ftraverse ftr)
 {
 	if(IS_SWEEP(ftr)) {
 		knh_Object_free(ctx, o);
@@ -649,6 +751,13 @@ void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, f_traverse ftr)
 		}
 		break;
 
+	case CLASS_Any:
+		{
+			knh_COBJ_t *c = (knh_COBJ_t*)o;
+			c->cfree(ctx, c->cptr, ftr);
+		}
+		break;
+
 	case CLASS_Nue :
 		{
 			knh_Nue_t *v = (knh_Nue_t*)o;
@@ -699,6 +808,14 @@ void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, f_traverse ftr)
 		}
 		break;
 
+	case CLASS_DictIdx :
+		{
+			knh_DictIdx_t *d = (knh_DictIdx_t*)o;
+			ftr(ctx, UP(d->terms));
+			ftr(ctx, UP(d->termsDictSet));
+		}
+		break;
+
 	case CLASS_Regex :
 		{
 			knh_Regex_traverse(ctx, (knh_Regex_t*)o, ftr);
@@ -804,9 +921,52 @@ knh_Int_t *new_Int0_(knh_int_t n)
 
 /* ------------------------------------------------------------------------ */
 
-#define _new_Int(ctx, n)      new_Int__fast(ctx, CLASS_Int, (n))
+#define _new_Int(ctx, n)      new_Int__fast(ctx, (n))
 
-KNHAPI(Int*) new_Int__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value)
+KNHAPI_(Int*) new_Int__fast(Ctx *ctx, knh_integer_t value)
+{
+	KNH_ASSERT(ctx != NULL);
+	if(KNH_TINT_MIN <= value && value <= KNH_TINT_MAX) {
+		return knh_tInt[value - (KNH_TINT_MIN)];
+	}
+
+	if(ctx->unusedObject == NULL) {
+		KNH_ASSERT(ctx->unusedObjectSize == 0);
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
+		if(ctx->unusedObject == NULL) {
+			KNH_EXIT("Enlarge KNH_TOBJECTLIST_SIZE %d x %d",
+					KNH_TOBJECTLIST_SIZE, KNH_TOBJECT_SIZE);
+			return NULL;
+		}
+		else {
+			((knh_Context_t*)ctx)->unusedObjectSize += KNH_TOBJECT_SIZE;
+		}
+	}
+	{
+		knh_Object_t *o = ctx->unusedObject;
+		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
+		((knh_Context_t*)ctx)->unusedObjectSize -= 1;
+
+#ifdef KNH_STATMODE1
+		knh_statUsedObjectSize++;
+#endif
+		o->h.magic = KNH_OBJECT_MAGIC;
+#ifdef KNH_HOBJECT_REFC
+		o->h.refc = KNH_HOBJECT_REFC_INIT;
+#endif
+		o->h.flag = FLAG_Int;
+		o->h.bcid = CLASS_Int;
+		o->h.cid  = CLASS_Int;
+		knh_Int_t *v = (knh_Int_t*)o;
+		v->value = value;
+		return v;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+
+KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value)
 {
 	KNH_ASSERT(ctx != NULL);
 	if(cid == CLASS_Int && KNH_TINT_MIN <= value && value <= KNH_TINT_MAX) {
@@ -848,9 +1008,9 @@ KNHAPI(Int*) new_Int__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value)
 
 /* ------------------------------------------------------------------------ */
 
-#define _new_Float(ctx, n)      new_Float__fast(ctx, CLASS_Float, (n))
+#define _new_Float(ctx, n)      new_Float__fast(ctx, (n))
 
-KNHAPI(Float*) new_Float__fast(Ctx *ctx, knh_class_t cid, knh_float_t value)
+KNHAPI_(Float*) new_Float__fast(Ctx *ctx, knh_float_t value)
 {
 	KNH_ASSERT(ctx != NULL);
 	if(ctx->unusedObject == NULL) {
@@ -879,7 +1039,7 @@ KNHAPI(Float*) new_Float__fast(Ctx *ctx, knh_class_t cid, knh_float_t value)
 #endif
 		o->h.flag = FLAG_Float;
 		o->h.bcid = CLASS_Float;
-		o->h.cid  = cid;
+		o->h.cid  = CLASS_Float;
 		knh_Float_t *v = (knh_Float_t*)o;
 		v->value = value;
 		return v;
@@ -922,7 +1082,7 @@ knh_System_t *new_System0(void)
 
 	KNH_INITv(DP(o)->tConstDictMap, new_DictMap0(NULL, 256));
 
-	KNH_INITv(DP(o)->tfieldnDictIdx, new_DictIdx(NULL, KNH_TFIELDN_SIZE * 2, 0));
+	KNH_INITv(DP(o)->tfieldnDictIdx, new_DictIdx__ignoreCase(NULL, KNH_TFIELDN_SIZE * 2, 0));
 	KNH_INITv(DP(o)->tfileidDictIdx, new_DictIdx(NULL, 32, 0));
 	knh_DictIdx_add__fast(NULL, DP(o)->tfileidDictIdx, new_String__T(NULL, "(unknown)"));
 	KNH_INITv(DP(o)->tMethodFields, new_Array0(NULL, KNH_TMETHODFIELD_SIZE * 2));
@@ -1017,7 +1177,7 @@ void knh_tObject_init0(void)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_tObject_traverse(Ctx *ctx, f_traverse ftr)
+void knh_tObject_traverse(Ctx *ctx, knh_ftraverse ftr)
 {
 	/* TSTATICOBJ */ {
 		int i;
@@ -1029,7 +1189,7 @@ void knh_tObject_traverse(Ctx *ctx, f_traverse ftr)
 			}
 		}
 		if(IS_SWEEP(ftr)) {
-			KNH_FREE(knh_tStaticVariable, SIZEOF_TSTATICOBJ);
+			KNH_FREE(ctx, knh_tStaticVariable, SIZEOF_TSTATICOBJ);
 			knh_tStaticVariable = NULL;
 			knh_tStaticVariableSize = 0;
 		}
@@ -1040,7 +1200,7 @@ void knh_tObject_traverse(Ctx *ctx, f_traverse ftr)
 			ftr(ctx, UP(knh_tString[i]));
 		}
 		if(IS_SWEEP(ftr)) {
-			KNH_FREE(knh_tString, SIZEOF_TSTRING);
+			KNH_FREE(ctx, knh_tString, SIZEOF_TSTRING);
 			knh_tString = NULL;
 		}
 	}
@@ -1051,7 +1211,7 @@ void knh_tObject_traverse(Ctx *ctx, f_traverse ftr)
 			ftr(ctx, UP(knh_tInt[i - KNH_TINT_MIN]));
 		}
 		if(IS_SWEEP(ftr)) {
-			KNH_FREE(knh_tInt, SIZEOF_TINT);
+			KNH_FREE(ctx, knh_tInt, SIZEOF_TINT);
 			knh_tInt = NULL;
 		}
 	}
@@ -1071,7 +1231,7 @@ void knh_tObject_traverse(Ctx *ctx, f_traverse ftr)
 			knh_tObject_free(knh_tObjectList[i]);
 			knh_tObjectList[i] = NULL;
 		}
-		KNH_FREE(knh_tObjectList, SIZEOF_TOBJECTLIST);
+		KNH_FREE(ctx, knh_tObjectList, SIZEOF_TOBJECTLIST);
 		knh_tObjectList = NULL;
 	}
 }
